package dk.nsi.dds.projects.sxa.documentprovider.audit;

import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.nsi.hsuid._2016._08.hsuid_1_1.Attribute;
import dk.nsi.hsuid._2016._08.hsuid_1_1.HsuidHeader;
import dk.nsi.hsuid._2016._08.hsuid_1_1.NameConstraint;
import dk.nsi.hsuid._2016._08.hsuid_1_1.SubjectIdentifierType;
import loginfo.healthcare._2017_03_01.ClientIdentificationTypeEnumeration;
import loginfo.healthcare._2017_03_01.LogRecord;
import loginfo.healthcare._2017_03_01.RequestingClientType;
import loginfo.healthcare._2017_03_01.TraceInfoType;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetRequestType;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

public class LogRecordBuilder {
    private DateTimeProvider dateTimeProvider;

    public LogRecordBuilder(DateTimeProvider dateTimeProvider) {
        this.dateTimeProvider = dateTimeProvider;
    }

    private LogRecordBuilder extractSessionId(Header header) {
        throw new UnsupportedOperationException();
    }

    private LogRecordBuilder extractMessageId(Header header) {
        throw new UnsupportedOperationException();
    }

    private LogRecordBuilder extractRoute(LogRecord logRecord) {
        throw new UnsupportedOperationException();
    }

        private LogRecordBuilder extractUserCprNumber(HsuidHeader hsuidHeader) {
        throw new UnsupportedOperationException();
    }

    private LogRecordBuilder extractBehalfOfCprNumber(HsuidHeader hsuidHeader) {
        throw new UnsupportedOperationException();
    }

    private LogRecordBuilder extractLookupType(HsuidHeader hsuidHeader) {
        throw new UnsupportedOperationException();
    }

    private LogRecordBuilder extractCprNumber(RetrieveDocumentSetRequestType request) {
        throw new UnsupportedOperationException();
    }

    private LogRecordBuilder extractPrivacyOverrideCode(HsuidHeader hsuidHeader) {
        throw new UnsupportedOperationException();
    }

    public LogRecord buildLogRecord(HsuidHeader hsuidHeader, Header header, LogRecord forwardedLogRecord, RetrieveDocumentSetRequestType request) {
        LogRecord logRecord = new LogRecord();

        logRecord.setTraceInfo(buildTraceInfo(header, forwardedLogRecord));

        logRecord.setRequestingClient(buildRequestingClientType(hsuidHeader));

        return logRecord;
    }

    private TraceInfoType buildTraceInfo(Header header, LogRecord forwardedLogRecord) {
        TraceInfoType traceInfo = new TraceInfoType();

        traceInfo.setRequestTime(getRequestTime());
        traceInfo.setSessionId(header.getLinking().getFlowID());
        traceInfo.setRequestId(header.getLinking().getMessageID());

        return traceInfo;
    }

    private XMLGregorianCalendar getRequestTime() {
        try {
            return DatatypeFactory.newInstance().newXMLGregorianCalendar(dateTimeProvider.now().toGregorianCalendar());
        }
        catch(DatatypeConfigurationException e) {
            throw new IllegalStateException("Could not instantiate DatatypeFactory!", e);
        }
    }

    private RequestingClientType buildRequestingClientType(HsuidHeader hsuidHeader) {
        RequestingClientType requestingClientType = new RequestingClientType();

        requestingClientType.setClientIdentification(buildClientIdentification(hsuidHeader));
        requestingClientType.setClinicName("");
        requestingClientType.setClinicDepartment("");
        requestingClientType.setUserCprNumber(getActingUserCivilRegistrationNumber(hsuidHeader));
        requestingClientType.setUserName("");
        requestingClientType.setUserInitials("");
        requestingClientType.setBehalfOfCprNumber(getBehalfOfCprNumber(hsuidHeader));

        return requestingClientType;
    }

    private RequestingClientType.ClientIdentification buildClientIdentification(HsuidHeader hsuidHeader) {
        Attribute userTypeAttribute = getRequiredAttribute(hsuidHeader, NameConstraint.NSI_USERTYPE);
        switch(userTypeAttribute.getAttributeValue()) {
            case "nsi:Citizen":
                return buildCitizenClientIdentification(hsuidHeader);
            case "nsi:HealthcareProfessional":
                return buildHealthcareProfessionalClientIdentification(hsuidHeader);
            default:
                throw new IllegalArgumentException(String.format("Unexpected nsi:UserType: %s", userTypeAttribute.getAttributeValue()));
        }
    }

    private RequestingClientType.ClientIdentification buildCitizenClientIdentification(HsuidHeader hsuidHeader) {
        RequestingClientType.ClientIdentification clientIdentification = new RequestingClientType.ClientIdentification();

        clientIdentification.setClientIdentificationType(ClientIdentificationTypeEnumeration.CPR_NUMMER);
        clientIdentification.setValue(getActingUserCivilRegistrationNumber(hsuidHeader));

        return clientIdentification;
    }

    private RequestingClientType.ClientIdentification buildHealthcareProfessionalClientIdentification(HsuidHeader hsuidHeader) {
        RequestingClientType.ClientIdentification clientIdentification = new RequestingClientType.ClientIdentification();

        Attribute orgUsingIdAttribute = getRequiredAttribute(hsuidHeader, NameConstraint.NSI_ORG_USING_ID);
        clientIdentification.setClientIdentificationType(getClientIdentificationTypeForOrgnisation(orgUsingIdAttribute.getNameFormat()));
        clientIdentification.setValue(orgUsingIdAttribute.getAttributeValue());

        return clientIdentification;
    }

    private String getActingUserCivilRegistrationNumber(HsuidHeader hsuidHeader) {
        Attribute actingUserCivilRegistrationNumberAttribute = getRequiredAttribute(hsuidHeader, NameConstraint.NSI_ACTING_USER_CIVIL_REGISTRATION_NUMBER);
        return actingUserCivilRegistrationNumberAttribute.getAttributeValue();
    }

    private String getBehalfOfCprNumber(HsuidHeader hsuidHeader) {
        Attribute responsibleUserCivilRegistrationNumber = getOptionalAttribute(hsuidHeader, NameConstraint.NSI_RESPONSIBLE_USER_CIVIL_REGISTRATION_NUMBER);
        return responsibleUserCivilRegistrationNumber != null ? responsibleUserCivilRegistrationNumber.getAttributeValue() : null;
    }

    private ClientIdentificationTypeEnumeration getClientIdentificationTypeForOrgnisation(SubjectIdentifierType subjectIdentifier) {
        switch(subjectIdentifier) {
            case NSI_SKSCODE:
                return ClientIdentificationTypeEnumeration.SKS;
            case NSI_SORCODE:
                return ClientIdentificationTypeEnumeration.SOR;
            case NSI_YNUMBER:
                return ClientIdentificationTypeEnumeration.YDERNUMMER;
            default:
                throw new IllegalArgumentException(String.format("Unexpected SubjectIdentifierType: %s", subjectIdentifier));
        }
    }

    private Attribute getRequiredAttribute(HsuidHeader hsuidHeader, NameConstraint nameConstraint) {
        return getAttribute(hsuidHeader, nameConstraint, true);
    }

    private Attribute getOptionalAttribute(HsuidHeader hsuidHeader, NameConstraint nameConstraint) {
        return getAttribute(hsuidHeader, nameConstraint, false);
    }

    private Attribute getAttribute(HsuidHeader hsuidHeader, NameConstraint nameConstraint, boolean required) {
        for(Attribute a : hsuidHeader.getAssertion().getAttributeStatement().getAttribute()) {
            if(a.getName().equals(nameConstraint)) {
                return a;
            }
        }

        if(required) {
            throw new IllegalArgumentException(String.format("No attribute matched name constraint %s", nameConstraint.toString()));
        }

        return null;
    }
}
