package dk.nsi.dds.projects.sxa.documentprovider.audit;

import dk.nsi.dds.projects.sxa.documentprovider.context.HttpHeaderContext;
import dk.nsi.hsuid._2016._08.hsuid_1_1.HsuidHeader;
import loginfo.healthcare._2017_03_01.LogRecord;
import org.apache.cxf.binding.soap.SoapHeader;
import org.apache.cxf.headers.Header;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetRequestType;
import org.w3c.dom.Node;

import javax.inject.Inject;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.ws.WebServiceContext;
import java.io.IOException;
import java.io.StringWriter;
import java.util.List;

public class LogRecordHeaderInterceptor {
    public static final String LOGRECORD_HEADER_NAME = "healthcare.loginfo.logdata";

    public static final String HSUID_HEADER_NAMESPACE = "http://www.nsi.dk/hsuid/2016/08/hsuid-1.1.xsd";
    public static final String HSUID_HEADER_NAME = "HsuidHeader";

    public static final String MEDCOM_HEADER_NAMESPACE = "http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd";
    public static final String MEDCOM_HEADER_NAME = "Header";

    @Inject
    private WebServiceContext webServiceContext;

    @Inject
    private HttpHeaderContext httpHeaderContext;

    @Inject
    private LogRecordBuilder logRecordBuilder;

    @AroundInvoke
    public Object generateLogRecordHeader(InvocationContext context) throws Exception {
        // Extract required information from headers, request body, etc.
        HsuidHeader hsuidHeader = extractHsuidHeader();
        dk.medcom.dgws._2006._04.dgws_1_0.Header header = extractMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Generate the LogRecord
        LogRecord logRecord = logRecordBuilder.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Serialize the LogRecord, put it on the header context
        serializeAndAddToContext(logRecord);

        return context.proceed();
    }

    private HsuidHeader extractHsuidHeader() throws Exception {
        List<SoapHeader> soapHeaders = (List<SoapHeader>) webServiceContext.getMessageContext().get(Header.HEADER_LIST);

        for(SoapHeader sh : soapHeaders) {
            if(sh.getName().getNamespaceURI().equals(HSUID_HEADER_NAMESPACE) && sh.getName().getLocalPart().equals(HSUID_HEADER_NAME)) {
                return (HsuidHeader) JAXBContext.newInstance(HsuidHeader.class).createUnmarshaller().unmarshal((Node) sh.getObject());
            }
        }

        throw new IllegalStateException("No HsuidHeader found in request!");
    }

    private dk.medcom.dgws._2006._04.dgws_1_0.Header extractMedcomHeader() throws Exception {
        List<SoapHeader> soapHeaders = (List<SoapHeader>) webServiceContext.getMessageContext().get(Header.HEADER_LIST);

        for(SoapHeader sh : soapHeaders) {
            if(sh.getName().getNamespaceURI().equals(MEDCOM_HEADER_NAMESPACE) && sh.getName().getLocalPart().equals(MEDCOM_HEADER_NAME)) {
                return (dk.medcom.dgws._2006._04.dgws_1_0.Header) JAXBContext.newInstance(dk.medcom.dgws._2006._04.dgws_1_0.Header.class).createUnmarshaller().unmarshal((Node) sh.getObject());
            }
        }

        throw new IllegalStateException("No HsuidHeader found in request!");
    }

    private void serializeAndAddToContext(LogRecord logRecord) throws Exception {
        try(StringWriter writer = new StringWriter()) {
            JAXBContext.newInstance(LogRecord.class).createMarshaller().marshal(logRecord, writer);
            httpHeaderContext.addHttpHeader(LOGRECORD_HEADER_NAME, writer.toString());
        }
    }
}
