package dk.nsi.dds.projects.sxa.documentprovider.audit;

import com.sun.org.apache.xerces.internal.jaxp.datatype.XMLGregorianCalendarImpl;
import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.medcom.dgws._2006._04.dgws_1_0.Linking;
import dk.nsi.dds.projects.sxa.common.HsuidHeaderUtil;
import dk.nsi.hsuid._2016._08.hsuid_1_1.HsuidHeader;
import loginfo.healthcare._2017_03_01.ClientIdentificationTypeEnumeration;
import loginfo.healthcare._2017_03_01.LogRecord;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetRequestType;
import sun.awt.image.ImageWatched;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import static org.junit.Assert.*;

@RunWith(MockitoJUnitRunner.class)
public class LogRecordBuilderTest {
    private LogRecordBuilder subject;

    @Mock
    private DateTimeProvider dateTimeProvider;

    @Before
    public void setup() {
        Mockito.when(dateTimeProvider.now()).thenReturn(DateTime.parse("2020-10-15T10:14:22.854+02:00"));

        subject = new LogRecordBuilder(dateTimeProvider);
    }

    @Test
    public void buildLogRecord_traceinfo_requestTime() throws DatatypeConfigurationException {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals(DatatypeFactory.newInstance().newXMLGregorianCalendar("2020-10-15T10:14:22.854+02:00"), logRecord.getTraceInfo().getRequestTime());
    }

    @Test
    public void buildLogRecord_traceinfo_sessionId() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // "11111111-1111-1111-1111-111111111111"
        // Assert
        assertEquals("00000000-0000-0000-0000-000000000000", logRecord.getTraceInfo().getSessionId());
    }

    @Test
    public void buildLogRecord_traceinfo_requestId() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals("11111111-1111-1111-1111-111111111111", logRecord.getTraceInfo().getRequestId());
    }

    @Test
    public void buildLogRecord_clientIdentificationType_citizen() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals(ClientIdentificationTypeEnumeration.CPR_NUMMER, logRecord.getRequestingClient().getClientIdentification().getClientIdentificationType());
        assertEquals("0987654321", logRecord.getRequestingClient().getClientIdentification().getValue());
    }

    @Test
    public void buildLogRecord_clientIdentificationType_healthcareProfessional_sks() {
        // Arrange
        String organisationIdentifier = "987123";
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getSksHealthcareProfessionalHsuidHeader(organisationIdentifier);
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals(ClientIdentificationTypeEnumeration.SKS, logRecord.getRequestingClient().getClientIdentification().getClientIdentificationType());
        assertEquals(organisationIdentifier, logRecord.getRequestingClient().getClientIdentification().getValue());
    }

    @Test
    public void buildLogRecord_clientIdentificationType_healthcareProfessional_sor() {
        // Arrange
        String organisationIdentifier = "440081000016006";
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getSorHealthcareProfessionalHsuidHeader(organisationIdentifier);
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals(ClientIdentificationTypeEnumeration.SOR, logRecord.getRequestingClient().getClientIdentification().getClientIdentificationType());
        assertEquals(organisationIdentifier, logRecord.getRequestingClient().getClientIdentification().getValue());
    }

    @Test
    public void buildLogRecord_clientIdentificationType_healthcareProfessional_yder() {
        // Arrange
        String organisationIdentifier = "445566";
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getYderHealthcareProfessionalHsuidHeader(organisationIdentifier);
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals(ClientIdentificationTypeEnumeration.YDERNUMMER, logRecord.getRequestingClient().getClientIdentification().getClientIdentificationType());
        assertEquals(organisationIdentifier, logRecord.getRequestingClient().getClientIdentification().getValue());
    }

    @Test
    public void buildLogRecord_requestingClient_clinicName() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals("", logRecord.getRequestingClient().getClinicName());
    }

    @Test
    public void buildLogRecord_requestingClient_clinicDepartment() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals("", logRecord.getRequestingClient().getClinicDepartment());
    }

    @Test
    public void buildLogRecord_requestingClient_userCprNumber() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals("0987654321", logRecord.getRequestingClient().getUserCprNumber());
    }

    @Test
    public void buildLogRecord_requestingClient_username() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals("", logRecord.getRequestingClient().getUserName());
    }

    @Test
    public void buildLogRecord_requestingClient_userInitials() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals("", logRecord.getRequestingClient().getUserInitials());
    }

    @Test
    public void buildLogRecord_requestingClient_behalfOfCprNumber() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getBehalfOfCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertEquals("9876543210", logRecord.getRequestingClient().getBehalfOfCprNumber());
    }

    @Test
    public void buildLogRecord_requestingClient_behalfOfCprNumber_none() {
        // Arrange
        HsuidHeader hsuidHeader = HsuidHeaderUtil.getCitizenHsuidHeader();
        Header header = getMedcomHeader();
        LogRecord forwardedLogRecord = null;
        RetrieveDocumentSetRequestType request = null;

        // Act
        LogRecord logRecord = subject.buildLogRecord(hsuidHeader, header, forwardedLogRecord, request);

        // Assert
        assertNull(logRecord.getRequestingClient().getBehalfOfCprNumber());
    }

    private Header getMedcomHeader() {
        Header header = new Header();

        Linking linking = new Linking();
        linking.setFlowID("00000000-0000-0000-0000-000000000000");
        linking.setMessageID("11111111-1111-1111-1111-111111111111");

        header.setLinking(linking);

        return header;
    }
}