package dk.nsi.dds.projects.sxa.documentprovider.audit;

import dk.medcom.dgws._2006._04.dgws_1_0.Linking;
import dk.nsi.dds.projects.sxa.documentprovider.context.HttpHeaderContext;
import dk.nsi.dds.projects.sxa.documentprovider.util.HsuidHeaderUtil;
import dk.nsi.hsuid.*;
import dk.nsi.hsuid._2016._08.hsuid_1_1.HsuidHeader;
import loginfo.healthcare._2017_03_01.LogRecord;
import org.apache.cxf.binding.soap.SoapHeader;
import org.apache.cxf.headers.Header;
import org.apache.cxf.jaxb.JAXBDataBinding;
import org.dom4j.dom.DOMDocument;
import org.jglue.cdiunit.CdiRunner;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.enterprise.inject.Produces;
import javax.inject.Inject;
import javax.interceptor.InvocationContext;
import javax.xml.bind.JAXBContext;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;

@RunWith(CdiRunner.class)
public class LogRecordHeaderInterceptorTest {
    @Inject
    private LogRecordHeaderInterceptor subject;

    @Produces
    @Mock
    private HttpHeaderContext httpHeaderContext;

    @Produces
    @Mock
    private WebServiceContext webServiceContext;

    @Produces
    @Mock
    private LogRecordBuilder logRecordBuilder;

    @Mock
    private InvocationContext invocationContext;

    @Before
    public void setup() {
        Mockito.when(logRecordBuilder.buildLogRecord(Mockito.any(), Mockito.any(), Mockito.any(), Mockito.any())).thenReturn(new LogRecord());
    }

    @Test
    public void generateLogRecordHeader_headerIsOnHeaderContext() throws Exception {
        // Arrange
        Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        JAXBContext.newInstance(HsuidHeader.class).createMarshaller().marshal(HsuidHeaderUtil.getCitizenHsuidHeader(), doc);

        Document medcomDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        JAXBContext.newInstance(dk.medcom.dgws._2006._04.dgws_1_0.Header.class).createMarshaller().marshal(getMedcomHeader(), medcomDoc);

        List<SoapHeader> soapHeaders = new ArrayList<>();
        soapHeaders.add(new SoapHeader(new QName(LogRecordHeaderInterceptor.HSUID_HEADER_NAMESPACE, LogRecordHeaderInterceptor.HSUID_HEADER_NAME, ""), doc, new JAXBDataBinding(HsuidHeader.class)));
        soapHeaders.add(new SoapHeader(new QName(LogRecordHeaderInterceptor.MEDCOM_HEADER_NAMESPACE, LogRecordHeaderInterceptor.MEDCOM_HEADER_NAME, ""), medcomDoc, new JAXBDataBinding(dk.medcom.dgws._2006._04.dgws_1_0.Header.class)));
        setupMocks(soapHeaders);

        // Act
        Object result = subject.generateLogRecordHeader(invocationContext);

        // Assert
        Mockito.verify(httpHeaderContext).addHttpHeader(Mockito.anyString(), Mockito.anyString());
    }

    @Test(expected = IllegalStateException.class)
    public void generateLogRecordHeader_missingHsuidHeader() throws Exception {
        // Arrange
        setupMocks(new ArrayList<>());

        // Act
        subject.generateLogRecordHeader(invocationContext);

        // Assert
    }

    private void setupMocks(List<SoapHeader> soapHeaders) {
        MessageContext messageContext = Mockito.mock(MessageContext.class);
        Mockito.when(messageContext.get(Header.HEADER_LIST)).thenReturn(soapHeaders);

        Mockito.when(webServiceContext.getMessageContext()).thenReturn(messageContext);
    }

    private dk.medcom.dgws._2006._04.dgws_1_0.Header getMedcomHeader() {
        dk.medcom.dgws._2006._04.dgws_1_0.Header header = new dk.medcom.dgws._2006._04.dgws_1_0.Header();

        header.setLinking(new Linking());
        header.getLinking().setFlowID("00000000-0000-0000-0000-000000000000");
        header.getLinking().setMessageID("11111111-1111-1111-1111-111111111111");

        return header;
    }
}